/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "console/console.h"
#include "server/server.h"
#include "util/string.h"
#include "assert/assert.h"

bool_t exitRequested = false;

void cmdExit(const consolecmdexec_t *exec) {
  exitRequested = true;
}

void cmdServe(const consolecmdexec_t *exec) {
  uint16_t port = 3030;
  if(exec->argc != 0) {
    if(!stringToU16(exec->argv[0], &port)) {
      consolePrint("Invalid port number: %s", exec->argv[0]);
      return;
    }
  }

  errorret_t ret = serverStart((serverstart_t){
    .type = SERVER_TYPE_NETWORKED,
    .networked = {
      .port = 3030
    }
  });
  
  if(ret != ERROR_OK) {
    consolePrint("Failed to start server: %s", errorString());
    errorFlush();
    return;
  }
}

void cmdClose(const consolecmdexec_t *exec) {
  serverStop();
}

int main(void) {
  assertInit();
  consoleInit();
  serverInit();

  consoleRegCmd("exit", cmdExit);
  consoleRegCmd("serve", cmdServe);
  consoleRegCmd("close", cmdClose);

  InitWindow(1280, 720, DUSK_NAME);

  while(!WindowShouldClose()) {
    consoleUpdate();

    BeginDrawing();
    ClearBackground(RAYWHITE);

    consoleDraw();

    EndDrawing();

    if(exitRequested) break;
  }

  CloseWindow();
  serverDispose();
  
  return EXIT_SUCCESS;
}